@@ -1,13 +1,9 @@
 # -*- test-case-name: twisted.test.test_ftp -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 An FTP protocol implementation
-
-@author: Itamar Shtull-Trauring
-@author: Jp Calderone
-@author: Andrew Bennetts
 """
 
 # System Imports
@@ -49,18 +45,19 @@ CMD_OK                                  = "200.1"
 TYPE_SET_OK                             = "200.2"
 ENTERING_PORT_MODE                      = "200.3"
 CMD_NOT_IMPLMNTD_SUPERFLUOUS            = "202"
-SYS_STATUS_OR_HELP_REPLY                = "211"
+SYS_STATUS_OR_HELP_REPLY                = "211.1"
+FEAT_OK                                 = '211.2'
 DIR_STATUS                              = "212"
 FILE_STATUS                             = "213"
 HELP_MSG                                = "214"
 NAME_SYS_TYPE                           = "215"
 SVC_READY_FOR_NEW_USER                  = "220.1"
 WELCOME_MSG                             = "220.2"
-SVC_CLOSING_CTRL_CNX                    = "221"
-GOODBYE_MSG                             = "221"
+SVC_CLOSING_CTRL_CNX                    = "221.1"
+GOODBYE_MSG                             = "221.2"
 DATA_CNX_OPEN_NO_XFR_IN_PROGRESS        = "225"
-CLOSING_DATA_CNX                        = "226"
-TXFR_COMPLETE_OK                        = "226"
+CLOSING_DATA_CNX                        = "226.1"
+TXFR_COMPLETE_OK                        = "226.2"
 ENTERING_PASV_MODE                      = "227"
 ENTERING_EPSV_MODE                      = "229"
 USR_LOGGED_IN_PROCEED                   = "230.1"     # v1 of code 230
@@ -84,7 +81,8 @@ REQ_ACTN_ABRTD_INSUFF_STORAGE           = "452"
 
 SYNTAX_ERR                              = "500"
 SYNTAX_ERR_IN_ARGS                      = "501"
-CMD_NOT_IMPLMNTD                        = "502"
+CMD_NOT_IMPLMNTD                        = "502.1"
+OPTS_NOT_IMPLEMENTED                    = '502.2'
 BAD_CMD_SEQ                             = "503"
 CMD_NOT_IMPLMNTD_FOR_PARAM              = "504"
 NOT_LOGGED_IN                           = "530.1"     # v1 of code 530 - please log in
@@ -115,12 +113,14 @@ RESPONSE = {
     ENTERING_PORT_MODE:                 '200 PORT OK',
     CMD_NOT_IMPLMNTD_SUPERFLUOUS:       '202 Command not implemented, superfluous at this site',
     SYS_STATUS_OR_HELP_REPLY:           '211 System status reply',
+    FEAT_OK:                            ['211-Features:','211 End'],
     DIR_STATUS:                         '212 %s',
     FILE_STATUS:                        '213 %s',
     HELP_MSG:                           '214 help: %s',
     NAME_SYS_TYPE:                      '215 UNIX Type: L8',
     WELCOME_MSG:                        "220 %s",
     SVC_READY_FOR_NEW_USER:             '220 Service ready',
+    SVC_CLOSING_CTRL_CNX:               '221 Service closing control connection',
     GOODBYE_MSG:                        '221 Goodbye.',
     DATA_CNX_OPEN_NO_XFR_IN_PROGRESS:   '225 data connection open, no transfer in progress',
     CLOSING_DATA_CNX:                   '226 Abort successful',
@@ -134,9 +134,9 @@ RESPONSE = {
     MKD_REPLY:                          '257 "%s" created',
 
     # -- 300's --
-    'userotp':                          '331 Response to %s.',  # ???
     USR_NAME_OK_NEED_PASS:              '331 Password required for %s.',
     GUEST_NAME_OK_NEED_EMAIL:           '331 Guest login ok, type your email address as password.',
+    NEED_ACCT_FOR_LOGIN:                '332 Need account for login.',
 
     REQ_FILE_ACTN_PENDING_FURTHER_INFO: '350 Requested file action pending further information.',
 
@@ -146,13 +146,15 @@ RESPONSE = {
     CANT_OPEN_DATA_CNX:                 "425 Can't open data connection.",
     CNX_CLOSED_TXFR_ABORTED:            '426 Transfer aborted.  Data connection closed.',
 
+    REQ_ACTN_ABRTD_FILE_UNAVAIL:        '450 Requested action aborted. File unavailable.',
     REQ_ACTN_ABRTD_LOCAL_ERR:           '451 Requested action aborted. Local error in processing.',
-
+    REQ_ACTN_ABRTD_INSUFF_STORAGE:      '452 Requested action aborted. Insufficient storage.',
 
     # -- 500's --
     SYNTAX_ERR:                         "500 Syntax error: %s",
     SYNTAX_ERR_IN_ARGS:                 '501 syntax error in argument(s) %s.',
     CMD_NOT_IMPLMNTD:                   "502 Command '%s' not implemented",
+    OPTS_NOT_IMPLEMENTED:               "502 Option '%s' not implemented.",
     BAD_CMD_SEQ:                        '503 Incorrect sequence of commands: %s',
     CMD_NOT_IMPLMNTD_FOR_PARAM:         "504 Not implemented for parameter '%s'.",
     NOT_LOGGED_IN:                      '530 Please login with USER and PASS.',
@@ -165,6 +167,7 @@ RESPONSE = {
     FILE_EXISTS:                        '550 %s: File exists',
     IS_A_DIR:                           '550 %s: is a directory',
     REQ_ACTN_NOT_TAKEN:                 '550 Requested action not taken: %s',
+    PAGE_TYPE_UNK:                      '551 Page type unknown',
     EXCEEDED_STORAGE_ALLOC:             '552 Requested file action aborted, exceeded file storage allocation',
     FILENAME_NOT_ALLOWED:               '553 Requested action not taken, file name not allowed'
 }
@@ -221,6 +224,37 @@ def errnoToFailure(e, path):
         return defer.fail()
 
 
+def _isGlobbingExpression(segments=None):
+    """
+    Helper for checking if a FTPShell `segments` contains a wildcard Unix
+    expression.
+
+    Only filename globbing is supported.
+    This means that wildcards can only be presents in the last element of
+    `segments`.
+
+    @type  segments: C{list}
+    @param segments: List of path elements as used by the FTP server protocol.
+
+    @rtype: Boolean
+    @return: True if `segments` contains a globbing expression.
+    """
+    if not segments:
+        return False
+
+    # To check that something is a glob expression, we convert it to
+    # Regular Expression. If the result is the same as the original expression
+    # then it contains no globbing expression.
+    globCandidate = segments[-1]
+    # A set of default regex rules is added to all strings.
+    emtpyTranslations = fnmatch.translate('')
+    globTranslations = fnmatch.translate(globCandidate)
+
+    if globCandidate + emtpyTranslations == globTranslations:
+        return False
+    else:
+        return True
+
 
 class FTPCmdError(Exception):
     """
@@ -252,9 +286,6 @@ class AnonUserDeniedError(FTPCmdError):
     Raised when an anonymous user issues a command that will alter the
     filesystem
     """
-    def __init__(self):
-        # No message
-        FTPCmdError.__init__(self, None)
 
     errorCode = ANON_USER_DENIED
 
@@ -508,7 +539,9 @@ class DTPFactory(protocol.ClientFactory):
 
     # -- class variables --
     def __init__(self, pi, peerHost=None, reactor=None):
-        """Constructor
+        """
+        Constructor
+
         @param pi: this factory's protocol interpreter
         @param peerHost: if peerCheck is True, this is the tuple that the
             generated instance will use to perform security checks
@@ -661,6 +694,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
     dtpPort = None
     dtpInstance = None
     binary = True
+    PUBLIC_COMMANDS = ['FEAT', 'QUIT']
+    FEATURES = ['FEAT', 'MDTM', 'PASV', 'SIZE', 'TYPE A;I']
 
     passivePortRange = xrange(0, 1)
 
@@ -735,9 +770,19 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def processCommand(self, cmd, *params):
+
+        def call_ftp_command(command):
+            method = getattr(self, "ftp_" + command, None)
+            if method is not None:
+                return method(*params)
+            return defer.fail(CmdNotImplementedError(command))
+
         cmd = cmd.upper()
 
-        if self.state == self.UNAUTH:
+        if cmd in self.PUBLIC_COMMANDS:
+            return call_ftp_command(cmd)
+
+        elif self.state == self.UNAUTH:
             if cmd == 'USER':
                 return self.ftp_USER(*params)
             elif cmd == 'PASS':
@@ -752,10 +797,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
                 return BAD_CMD_SEQ, "PASS required after USER"
 
         elif self.state == self.AUTHED:
-            method = getattr(self, "ftp_" + cmd, None)
-            if method is not None:
-                return method(*params)
-            return defer.fail(CmdNotImplementedError(cmd))
+            return call_ftp_command(cmd)
 
         elif self.state == self.RENAMING:
             if cmd == 'RNTO':
@@ -833,7 +875,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_PASV(self):
-        """Request for a passive connection
+        """
+        Request for a passive connection
 
         from the rfc::
 
@@ -892,17 +935,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         if self.dtpInstance is None or not self.dtpInstance.isConnected:
             return defer.fail(BadCmdSequenceError('must send PORT or PASV before RETR'))
 
-        # bug in konqueror
-        if path == "-a":
-            path = ''
-        # bug in gFTP 2.0.15
-        if path == "-aL":
-            path = ''
-        # bug in Nautilus 2.10.0
-        if path == "-L":
-            path = ''
-        # bug in ange-ftp
-        if path == "-la":
+        # Various clients send flags like -L or -al etc.  We just ignore them.
+        if path.lower() in ['-a', '-l', '-la', '-al']:
             path = ''
 
         def gotListing(results):
@@ -950,29 +984,27 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         except InvalidPath:
             return defer.fail(FileNotFoundError(path))
 
-        def cbList(results):
+        def cbList(results, glob):
             """
-            Send, line by line, each file in the directory listing, and then
-            close the connection.
+            Send, line by line, each matching file in the directory listing, and
+            then close the connection.
 
             @type results: A C{list} of C{tuple}. The first element of each
                 C{tuple} is a C{str} and the second element is a C{list}.
             @param results: The names of the files in the directory.
 
-            @rtype: C{tuple}
+            @param glob: A shell-style glob through which to filter results (see
+                U{http://docs.python.org/2/library/fnmatch.html}), or C{None}
+                for no filtering.
+            @type glob: L{str} or L{NoneType}
+
             @return: A C{tuple} containing the status code for a successful
                 transfer.
+            @rtype: C{tuple}
             """
             self.reply(DATA_CNX_ALREADY_OPEN_START_XFR)
             for (name, ignored) in results:
-                self.dtpInstance.sendLine(name)
-            self.dtpInstance.transport.loseConnection()
-            return (TXFR_COMPLETE_OK,)
-
-        def cbGlob(results):
-            self.reply(DATA_CNX_ALREADY_OPEN_START_XFR)
-            for (name, ignored) in results:
-                if fnmatch.fnmatch(name, segments[-1]):
+                if not glob or (glob and fnmatch.fnmatch(name, glob)):
                     self.dtpInstance.sendLine(name)
             self.dtpInstance.transport.loseConnection()
             return (TXFR_COMPLETE_OK,)
@@ -987,24 +1019,24 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
                 occurred while trying to list the contents of a nonexistent
                 directory.
 
-            @rtype: C{tuple}
             @returns: A C{tuple} containing the status code for a successful
                 transfer.
+            @rtype: C{tuple}
             """
             self.dtpInstance.transport.loseConnection()
             return (TXFR_COMPLETE_OK,)
 
-        # XXX This globbing may be incomplete: see #4181
-        if segments and (
-            '*' in segments[-1] or '?' in segments[-1] or
-            ('[' in segments[-1] and ']' in segments[-1])):
-            d = self.shell.list(segments[:-1])
-            d.addCallback(cbGlob)
+        if _isGlobbingExpression(segments):
+            # Remove globbing expression from path
+            # and keep to be used for filtering.
+            glob = segments.pop()
         else:
-            d = self.shell.list(segments)
-            d.addCallback(cbList)
-            # self.shell.list will generate an error if the path is invalid
-            d.addErrback(listErr)
+            glob = None
+
+        d = self.shell.list(segments)
+        d.addCallback(cbList, glob)
+        # self.shell.list will generate an error if the path is invalid
+        d.addErrback(listErr)
         return d
 
 
@@ -1031,6 +1063,17 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_RETR(self, path):
+        """
+        This command causes the content of a file to be sent over the data
+        transfer channel. If the path is to a folder, an error will be raised.
+
+        @type path: C{str}
+        @param path: The path to the file which should be transferred over the
+        data transfer channel.
+
+        @rtype: L{Deferred}
+        @return: a L{Deferred} which will be fired when the transfer is done.
+        """
         if self.dtpInstance is None:
             raise BadCmdSequenceError('PORT or PASV required before RETR')
 
@@ -1061,6 +1104,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         def ebSent(err):
             log.msg("Unexpected error attempting to transmit file to client:")
             log.err(err)
+            if err.check(FTPCmdError):
+                return err
             return (CNX_CLOSED_TXFR_ABORTED,)
 
         def cbOpened(file):
@@ -1075,7 +1120,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
             return d
 
         def ebOpened(err):
-            if not err.check(PermissionDeniedError, FileNotFoundError, IsNotADirectoryError):
+            if not err.check(PermissionDeniedError, FileNotFoundError, IsADirectoryError):
                 log.msg("Unexpected error attempting to open file for transmission:")
                 log.err(err)
             if err.check(FTPCmdError):
@@ -1115,6 +1160,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         def ebSent(err):
             log.msg("Unexpected error receiving file from client:")
             log.err(err)
+            if err.check(FTPCmdError):
+                return err
             return (CNX_CLOSED_TXFR_ABORTED,)
 
         def cbConsumer(cons):
@@ -1155,6 +1202,31 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_SIZE(self, path):
+        """
+        File SIZE
+
+        The FTP command, SIZE OF FILE (SIZE), is used to obtain the transfer
+        size of a file from the server-FTP process.  This is the exact number
+        of octets (8 bit bytes) that would be transmitted over the data
+        connection should that file be transmitted.  This value will change
+        depending on the current STRUcture, MODE, and TYPE of the data
+        connection or of a data connection that would be created were one
+        created now.  Thus, the result of the SIZE command is dependent on
+        the currently established STRU, MODE, and TYPE parameters.
+
+        The SIZE command returns how many octets would be transferred if the
+        file were to be transferred using the current transfer structure,
+        mode, and type.  This command is normally used in conjunction with
+        the RESTART (REST) command when STORing a file to a remote server in
+        STREAM mode, to determine the restart point.  The server-PI might
+        need to read the partially transferred file, do any appropriate
+        conversion, and count the number of octets that would be generated
+        when sending the file in order to correctly respond to this command.
+        Estimates of the file transfer size MUST NOT be returned; only
+        precise information is acceptable.
+
+        http://tools.ietf.org/html/rfc3659
+        """
         try:
             newsegs = toSegments(self.workingDirectory, path)
         except InvalidPath:
@@ -1167,6 +1239,18 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_MDTM(self, path):
+        """
+        File Modification Time (MDTM)
+
+        The FTP command, MODIFICATION TIME (MDTM), can be used to determine
+        when a file in the server NVFS was last modified.  This command has
+        existed in many FTP servers for many years, as an adjunct to the REST
+        command for STREAM mode, thus is widely available.  However, where
+        supported, the "modify" fact that can be provided in the result from
+        the new MLST command is recommended as a superior alternative.
+
+        http://tools.ietf.org/html/rfc3659
+        """
         try:
             newsegs = toSegments(self.workingDirectory, path)
         except InvalidPath:
@@ -1179,6 +1263,18 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_TYPE(self, type):
+        """
+        REPRESENTATION TYPE (TYPE)
+
+        The argument specifies the representation type as described
+        in the Section on Data Representation and Storage.  Several
+        types take a second parameter.  The first parameter is
+        denoted by a single Telnet character, as is the second
+        Format parameter for ASCII and EBCDIC; the second parameter
+        for local byte is a decimal integer to indicate Bytesize.
+        The parameters are separated by a <SP> (Space, ASCII code
+        32).
+        """
         p = type.upper()
         if p:
             f = getattr(self, 'type_' + p[0], None)
@@ -1271,14 +1367,33 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         return self.shell.rename(fromsegs, tosegs).addCallback(lambda ign: (REQ_FILE_ACTN_COMPLETED_OK,))
 
 
+    def ftp_FEAT(self):
+        """
+        Advertise the features supported by the server.
+
+        http://tools.ietf.org/html/rfc2389
+        """
+        self.sendLine(RESPONSE[FEAT_OK][0])
+        for feature in self.FEATURES:
+            self.sendLine(' ' + feature)
+        self.sendLine(RESPONSE[FEAT_OK][1])
+
+    def ftp_OPTS(self, option):
+        """
+        Handle OPTS command.
+
+        http://tools.ietf.org/html/draft-ietf-ftpext-utf-8-option-00
+        """
+        return self.reply(OPTS_NOT_IMPLEMENTED, option)
+
     def ftp_QUIT(self):
         self.reply(GOODBYE_MSG)
         self.transport.loseConnection()
         self.disconnected = True
 
-
     def cleanupDTP(self):
-        """call when DTP connection exits
+        """
+        Call when DTP connection exits
         """
         log.msg('cleanupDTP', debug=True)
 
@@ -1520,7 +1635,8 @@ class IWriteFile(Interface):
         """
 
 def _getgroups(uid):
-    """Return the primary and supplementary groups for the given UID.
+    """
+    Return the primary and supplementary groups for the given UID.
 
     @type uid: C{int}
     """
@@ -2043,8 +2159,23 @@ class ProtocolWrapper(protocol.Protocol):
         self.deferred.callback(None)
 
 
+
+class IFinishableConsumer(interfaces.IConsumer):
+    """
+    A Consumer for producers that finish.
+
+    @since: 11.0
+    """
+
+    def finish():
+        """
+        The producer has finished producing.
+        """
+
+
+
 class SenderProtocol(protocol.Protocol):
-    implements(interfaces.IFinishableConsumer)
+    implements(IFinishableConsumer)
 
     def __init__(self):
         # Fired upon connection
@@ -2091,7 +2222,8 @@ class SenderProtocol(protocol.Protocol):
 
 
 def decodeHostPort(line):
-    """Decode an FTP response specifying a host and port.
+    """
+    Decode an FTP response specifying a host and port.
 
     @return: a 2-tuple of (host, port).
     """
@@ -2114,7 +2246,8 @@ def _unwrapFirstError(failure):
     return failure.value.subFailure
 
 class FTPDataPortFactory(protocol.ServerFactory):
-    """Factory for data connections that use the PORT command
+    """
+    Factory for data connections that use the PORT command
 
     (i.e. "active" transfers)
     """
@@ -2742,6 +2875,27 @@ class FTPClient(FTPClientBasic):
         return self.queueStringCommand('DELE ' + self.escapePath(path))
 
 
+    def removeDirectory(self, path):
+        """
+        Delete a directory on the server.
+
+        L{removeDirectory} issues a I{RMD} command to the server to remove the
+        indicated directory. Described in RFC959.
+
+        @param path: The path to the directory to delete. May be relative to
+            the current working directory.
+        @type path: C{str}
+
+        @return: A L{Deferred} which fires when the server responds. On error,
+            it is errbacked with either L{CommandFailed} or L{BadResponse}. On
+            success, it is called back with a list of response lines.
+        @rtype: L{Deferred}
+
+        @since: 11.1
+        """
+        return self.queueStringCommand('RMD ' + self.escapePath(path))
+
+
     def cdup(self):
         """
         Issues the CDUP (Change Directory UP) command.
@@ -2764,6 +2918,7 @@ class FTPClient(FTPClientBasic):
         """
         return self.queueStringCommand('PWD')
 
+
     def getDirectory(self):
         """
         Returns the current remote directory.
@@ -2799,7 +2954,8 @@ class FTPClient(FTPClientBasic):
 
 
 class FTPFileListProtocol(basic.LineReceiver):
-    """Parser for standard FTP file listings
+    """
+    Parser for standard FTP file listings
 
     This is the evil required to match::
 
@@ -2824,12 +2980,24 @@ class FTPFileListProtocol(basic.LineReceiver):
     date.  Check U{http://cr.yp.to/ftp.html} if you really want to try to parse
     it.
 
+    It also matches the following::
+        -rw-r--r--   1 root     other        531 Jan 29 03:26 I HAVE\ SPACE
+           - filename:   e.g. 'I HAVE SPACE'
+
+        -rw-r--r--   1 root     other        531 Jan 29 03:26 LINK -> TARGET
+           - filename:   e.g. 'LINK'
+           - linktarget: e.g. 'TARGET'
+
+        -rw-r--r--   1 root     other        531 Jan 29 03:26 N S -> L S
+           - filename:   e.g. 'N S'
+           - linktarget: e.g. 'L S'
+
     @ivar files: list of dicts describing the files in this listing
     """
     fileLinePattern = re.compile(
         r'^(?P<filetype>.)(?P<perms>.{9})\s+(?P<nlinks>\d*)\s*'
         r'(?P<owner>\S+)\s+(?P<group>\S+)\s+(?P<size>\d+)\s+'
-        r'(?P<date>...\s+\d+\s+[\d:]+)\s+(?P<filename>([^ ]|\\ )*?)'
+        r'(?P<date>...\s+\d+\s+[\d:]+)\s+(?P<filename>.{1,}?)'
         r'( -> (?P<linktarget>[^\r]*))?\r?$'
     )
     delimiter = '\n'
@@ -2845,7 +3013,8 @@ class FTPFileListProtocol(basic.LineReceiver):
             self.addFile(d)
 
     def parseDirectoryLine(self, line):
-        """Return a dictionary of fields, or None if line cannot be parsed.
+        """
+        Return a dictionary of fields, or None if line cannot be parsed.
 
         @param line: line of text expected to contain a directory entry
         @type line: str
@@ -2865,7 +3034,8 @@ class FTPFileListProtocol(basic.LineReceiver):
             return d
 
     def addFile(self, info):
-        """Append file information dictionary to the list of known files.
+        """
+        Append file information dictionary to the list of known files.
 
         Subclasses can override or extend this method to handle file
         information differently without affecting the parsing of data
@@ -2878,7 +3048,8 @@ class FTPFileListProtocol(basic.LineReceiver):
         self.files.append(info)
 
     def unknownLine(self, line):
-        """Deal with received lines which could not be parsed as file
+        """
+        Deal with received lines which could not be parsed as file
         information.
 
         Subclasses can override this to perform any special processing
@@ -2890,7 +3061,8 @@ class FTPFileListProtocol(basic.LineReceiver):
         pass
 
 def parsePWDResponse(response):
-    """Returns the path from a response to a PWD command.
+    """
+    Returns the path from a response to a PWD command.
 
     Responses typically look like::
 
