@@ -6,7 +6,7 @@
 """
 Tests for L{twisted.trial.reporter}.
 """
-
+from __future__ import division
 
 import errno, sys, os, re, StringIO
 from inspect import getmro
@@ -45,7 +45,7 @@ class BrokenStream(object):
         raise IOError(errno.EINTR, "Interrupted flush")
 
 
-class StringTest(unittest.TestCase):
+class StringTest(unittest.SynchronousTestCase):
     def stringComparison(self, expect, output):
         output = filter(None, output)
         self.failUnless(len(expect) <= len(output),
@@ -67,7 +67,7 @@ class StringTest(unittest.TestCase):
                                 % (exp,))
 
 
-class TestTestResult(unittest.TestCase):
+class TestTestResult(unittest.SynchronousTestCase):
     def setUp(self):
         self.result = reporter.TestResult()
 
@@ -118,21 +118,22 @@ class TestErrorReporting(StringTest):
     def test_formatErroredMethod(self):
         """
         A test method which runs and has an error recorded against it is
-        reported in the output stream with the I{ERROR} tag along with a summary
-        of what error was reported and the ID of the test.
+        reported in the output stream with the I{ERROR} tag along with a
+        summary of what error was reported and the ID of the test.
         """
-        suite = self.loader.loadClass(erroneous.TestFailureInSetUp)
+        cls = erroneous.SynchronousTestFailureInSetUp
+        suite = self.loader.loadClass(cls)
         output = self.getOutput(suite).splitlines()
         match = [
             self.doubleSeparator,
             '[ERROR]',
             'Traceback (most recent call last):',
             re.compile(r'^\s+File .*erroneous\.py., line \d+, in setUp$'),
-            re.compile(r'^\s+raise FoolishError, '
-                       r'.I am a broken setUp method.$'),
+            re.compile(r'^\s+raise FoolishError.'
+                       r'.I am a broken setUp method..$'),
             ('twisted.trial.test.erroneous.FoolishError: '
              'I am a broken setUp method'),
-            'twisted.trial.test.erroneous.TestFailureInSetUp.test_noop']
+            '%s.%s.test_noop' % (cls.__module__, cls.__name__)]
         self.stringComparison(match, output)
 
 
@@ -208,8 +209,7 @@ class TestErrorReporting(StringTest):
             re.compile('^\s+File .*erroneous\.py", line \d+, in go'),
             re.compile('^\s+raise RuntimeError\(self.hiddenExceptionMsg\)'),
             'exceptions.RuntimeError: something blew up',
-            'twisted.trial.test.erroneous.DelayedCall.testHiddenException',
-            ]
+            'twisted.trial.test.erroneous.DelayedCall.testHiddenException']
         self.stringComparison(match, output)
 
 
@@ -227,14 +227,23 @@ class TestUncleanWarningWrapperErrorReporting(TestErrorReporting):
 
 
 
-class TracebackHandling(unittest.TestCase):
+class TracebackHandling(unittest.SynchronousTestCase):
+
     def getErrorFrames(self, test):
+        """
+        Run the given C{test}, make sure it fails and return the trimmed
+        frames.
+
+        @param test: The test case to run.
+
+        @return: The C{list} of frames trimmed.
+        """
         stream = StringIO.StringIO()
         result = reporter.Reporter(stream)
         test.run(result)
         bads = result.failures + result.errors
-        assert len(bads) == 1
-        assert bads[0][0] == test
+        self.assertEqual(len(bads), 1)
+        self.assertEqual(bads[0][0], test)
         return result._trimFrames(bads[0][1].frames)
 
     def checkFrames(self, observedFrames, expectedFrames):
@@ -260,7 +269,11 @@ class TracebackHandling(unittest.TestCase):
                           ('subroutine', 'twisted/trial/test/erroneous')])
 
     def test_deferred(self):
-        test = erroneous.TestFailureInDeferredChain('test_fail')
+        """
+        C{_trimFrames} removes traces of C{_runCallbacks} when getting an error
+        in a callback returned by a C{TestCase} based test.
+        """
+        test = erroneous.TestAsynchronousFail('test_fail')
         frames = self.getErrorFrames(test)
         self.checkFrames(frames,
                          [('_later', 'twisted/trial/test/erroneous')])
@@ -273,6 +286,17 @@ class TracebackHandling(unittest.TestCase):
         result = reporter.Reporter(None)
         self.assertEqual(['fake frame'], result._trimFrames(['fake frame']))
 
+    def test_exception(self):
+        """
+        C{_trimFrames} removes traces of C{runWithWarningsSuppressed} from
+        C{_utilspy3} when a synchronous exception happens in a C{TestCase}
+        based test.
+        """
+        test = erroneous.TestAsynchronousFail('test_exception')
+        frames = self.getErrorFrames(test)
+        self.checkFrames(frames,
+                         [('test_exception', 'twisted/trial/test/erroneous')])
+
 
 class FormatFailures(StringTest):
     def setUp(self):
@@ -319,7 +343,7 @@ exceptions.TypeError: iterable argument required
         self.assertEqual(self.f.frames, frames)
 
 
-class PyunitTestNames(unittest.TestCase):
+class PyunitTestNames(unittest.SynchronousTestCase):
     def setUp(self):
         self.stream = StringIO.StringIO()
         self.test = sample.PyunitTest('test_foo')
@@ -328,7 +352,7 @@ class PyunitTestNames(unittest.TestCase):
         result = reporter.VerboseTextReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
-        self.failUnlessEqual(
+        self.assertEqual(
             output, 'twisted.trial.test.sample.PyunitTest.test_foo ... ')
 
     def test_treeReporter(self):
@@ -336,12 +360,12 @@ class PyunitTestNames(unittest.TestCase):
         result.startTest(self.test)
         output = self.stream.getvalue()
         output = output.splitlines()[-1].strip()
-        self.failUnlessEqual(output, result.getDescription(self.test) + ' ...')
+        self.assertEqual(output, result.getDescription(self.test) + ' ...')
 
     def test_getDescription(self):
         result = reporter.TreeReporter(self.stream)
         output = result.getDescription(self.test)
-        self.failUnlessEqual(output, 'test_foo')
+        self.assertEqual(output, 'test_foo')
 
 
     def test_minimalReporter(self):
@@ -360,7 +384,7 @@ class PyunitTestNames(unittest.TestCase):
         self.test.run(result)
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
-        self.failUnlessEqual(output[1:], ['1', '1', '0', '0', '0'])
+        self.assertEqual(output[1:], ['1', '1', '0', '0', '0'])
 
 
     def test_minimalReporterTime(self):
@@ -375,7 +399,7 @@ class PyunitTestNames(unittest.TestCase):
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
         timer = output[0]
-        self.assertEquals(timer, "0.7")
+        self.assertEqual(timer, "0.7")
 
 
     def test_emptyMinimalReporter(self):
@@ -386,11 +410,11 @@ class PyunitTestNames(unittest.TestCase):
         result = reporter.MinimalReporter(self.stream)
         result._printSummary()
         output = self.stream.getvalue().strip().split(' ')
-        self.failUnlessEqual(output, ['0', '0', '0', '0', '0', '0'])
+        self.assertEqual(output, ['0', '0', '0', '0', '0', '0'])
 
 
 
-class TestDirtyReactor(unittest.TestCase):
+class TestDirtyReactor(unittest.SynchronousTestCase):
     """
     The trial script has an option to treat L{DirtyReactorAggregateError}s as
     warnings, as a migration tool for test authors. It causes a wrapper to be
@@ -438,7 +462,7 @@ class TestDirtyReactor(unittest.TestCase):
         self.assertWarns(UserWarning, self.dirtyError.getErrorMessage(),
                          reporter.__file__,
                          result.addError, self.test, self.dirtyError)
-        self.assertEquals(result._originalReporter.errors, [])
+        self.assertEqual(result._originalReporter.errors, [])
 
 
     def test_dealsWithThreeTuples(self):
@@ -454,14 +478,14 @@ class TestDirtyReactor(unittest.TestCase):
         result.addError(self.test,
                         (self.dirtyError.type, self.dirtyError.value, None))
         self.assertEqual(len(result._originalReporter.errors), 1)
-        self.assertEquals(result._originalReporter.errors[0][1].type,
+        self.assertEqual(result._originalReporter.errors[0][1].type,
                           self.dirtyError.type)
-        self.assertEquals(result._originalReporter.errors[0][1].value,
+        self.assertEqual(result._originalReporter.errors[0][1].value,
                           self.dirtyError.value)
 
 
 
-class TrialTestNames(unittest.TestCase):
+class TrialTestNames(unittest.SynchronousTestCase):
 
     def setUp(self):
         self.stream = StringIO.StringIO()
@@ -471,14 +495,14 @@ class TrialTestNames(unittest.TestCase):
         result = reporter.VerboseTextReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
-        self.failUnlessEqual(output, self.test.id() + ' ... ')
+        self.assertEqual(output, self.test.id() + ' ... ')
 
     def test_treeReporter(self):
         result = reporter.TreeReporter(self.stream)
         result.startTest(self.test)
         output = self.stream.getvalue()
         output = output.splitlines()[-1].strip()
-        self.failUnlessEqual(output, result.getDescription(self.test) + ' ...')
+        self.assertEqual(output, result.getDescription(self.test) + ' ...')
 
     def test_treeReporterWithDocstrings(self):
         """A docstring"""
@@ -489,10 +513,10 @@ class TrialTestNames(unittest.TestCase):
     def test_getDescription(self):
         result = reporter.TreeReporter(self.stream)
         output = result.getDescription(self.test)
-        self.failUnlessEqual(output, "test_foo")
+        self.assertEqual(output, "test_foo")
 
 
-class TestSkip(unittest.TestCase):
+class TestSkip(unittest.SynchronousTestCase):
     """
     Tests for L{reporter.Reporter}'s handling of skips.
     """
@@ -513,7 +537,7 @@ class TestSkip(unittest.TestCase):
 
     def test_success(self):
         self.result.addSkip(self.test, 'some reason')
-        self.failUnlessEqual(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_summary(self):
@@ -526,7 +550,7 @@ class TestSkip(unittest.TestCase):
         output = self.stream.getvalue().splitlines()[-1]
         prefix = 'PASSED '
         self.failUnless(output.startswith(prefix))
-        self.failUnlessEqual(output[len(prefix):].strip(), '(skips=1)')
+        self.assertEqual(output[len(prefix):].strip(), '(skips=1)')
 
 
     def test_basicErrors(self):
@@ -537,7 +561,7 @@ class TestSkip(unittest.TestCase):
         self.result.addSkip(self.test, 'some reason')
         self.result.done()
         output = self.stream.getvalue().splitlines()[3]
-        self.failUnlessEqual(output.strip(), 'some reason')
+        self.assertEqual(output.strip(), 'some reason')
 
 
     def test_booleanSkip(self):
@@ -549,7 +573,7 @@ class TestSkip(unittest.TestCase):
         self.result.addSkip(self.test, True)
         self.result.done()
         output = self.stream.getvalue().splitlines()[3]
-        self.failUnlessEqual(output, 'True')
+        self.assertEqual(output, 'True')
 
 
     def test_exceptionSkip(self):
@@ -564,7 +588,7 @@ class TestSkip(unittest.TestCase):
         self.result.addSkip(self.test, error)
         self.result.done()
         output = '\n'.join(self.stream.getvalue().splitlines()[3:5]).strip()
-        self.failUnlessEqual(output, str(e))
+        self.assertEqual(output, str(e))
 
 
 class UncleanWarningSkipTest(TestSkip):
@@ -585,7 +609,7 @@ class UncleanWarningSkipTest(TestSkip):
 
 
 
-class TodoTest(unittest.TestCase):
+class TodoTest(unittest.SynchronousTestCase):
     """
     Tests for L{reporter.Reporter}'s handling of todos.
     """
@@ -735,7 +759,7 @@ class MockColorizer:
 
 
 
-class TestTreeReporter(unittest.TestCase):
+class TestTreeReporter(unittest.SynchronousTestCase):
     def setUp(self):
         self.test = sample.FooTest('test_foo')
         self.stream = StringIO.StringIO()
@@ -791,8 +815,8 @@ class TestTreeReporter(unittest.TestCase):
         """
         self.result.addSuccess(self.test)
         self.result.done()
-        self.assertEquals(self.log[1], (self.result.SUCCESS, 'PASSED'))
-        self.assertEquals(
+        self.assertEqual(self.log[1], (self.result.SUCCESS, 'PASSED'))
+        self.assertEqual(
             self.stream.getvalue().splitlines()[-1].strip(), "(successes=1)")
 
 
@@ -806,8 +830,8 @@ class TestTreeReporter(unittest.TestCase):
         except RuntimeError:
             self.result.addError(self, sys.exc_info())
         self.result.done()
-        self.assertEquals(self.log[1], (self.result.FAILURE, 'FAILED'))
-        self.assertEquals(
+        self.assertEqual(self.log[1], (self.result.FAILURE, 'FAILED'))
+        self.assertEqual(
             self.stream.getvalue().splitlines()[-1].strip(), "(errors=1)")
 
 
@@ -844,16 +868,17 @@ class TestTreeReporter(unittest.TestCase):
             extra = sample.FooTest('test_bar')
             self.result.addError(extra, sys.exc_info())
         self.result.done()
-	grouped = self.result._groupResults(
-	    self.result.errors, self.result._formatFailureTraceback)
-        self.assertEquals(grouped[0][1], [self, self.test])
-        self.assertEquals(grouped[1][1], [extra])
+        grouped = self.result._groupResults(
+            self.result.errors, self.result._formatFailureTraceback)
+        self.assertEqual(grouped[0][1], [self, self.test])
+        self.assertEqual(grouped[1][1], [extra])
 
 
     def test_printResults(self):
         """
         L{Reporter._printResults} uses the results list and formatter callable
-        passed to it to produce groups of results to write to its output stream.
+        passed to it to produce groups of results to write to its output
+        stream.
         """
         def formatter(n):
             return str(n) + '\n'
@@ -862,7 +887,7 @@ class TestTreeReporter(unittest.TestCase):
         third = sample.PyunitTest('test_foo')
         self.result._printResults(
             'FOO', [(first, 1), (second, 1), (third, 2)], formatter)
-        self.assertEquals(
+        self.assertEqual(
             self.stream.getvalue(),
             "%(double separator)s\n"
             "FOO\n"
@@ -883,7 +908,7 @@ class TestTreeReporter(unittest.TestCase):
 
 
 
-class TestReporterInterface(unittest.TestCase):
+class TestReporterInterface(unittest.SynchronousTestCase):
     """
     Tests for the bare interface of a trial reporter.
 
@@ -908,7 +933,7 @@ class TestReporterInterface(unittest.TestCase):
         """
         shouldStop is False to begin with.
         """
-        self.assertEquals(False, self.result.shouldStop)
+        self.assertEqual(False, self.result.shouldStop)
 
 
     def test_shouldStopTrueAfterStop(self):
@@ -916,14 +941,14 @@ class TestReporterInterface(unittest.TestCase):
         shouldStop becomes True soon as someone calls stop().
         """
         self.result.stop()
-        self.assertEquals(True, self.result.shouldStop)
+        self.assertEqual(True, self.result.shouldStop)
 
 
     def test_wasSuccessfulInitiallyTrue(self):
         """
         wasSuccessful() is True when there have been no results reported.
         """
-        self.assertEquals(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulTrueAfterSuccesses(self):
@@ -932,7 +957,7 @@ class TestReporterInterface(unittest.TestCase):
         otherwise.
         """
         self.result.addSuccess(self.test)
-        self.assertEquals(True, self.result.wasSuccessful())
+        self.assertEqual(True, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulFalseAfterErrors(self):
@@ -943,7 +968,7 @@ class TestReporterInterface(unittest.TestCase):
             1 / 0
         except ZeroDivisionError:
             self.result.addError(self.test, sys.exc_info())
-        self.assertEquals(False, self.result.wasSuccessful())
+        self.assertEqual(False, self.result.wasSuccessful())
 
 
     def test_wasSuccessfulFalseAfterFailures(self):
@@ -954,7 +979,7 @@ class TestReporterInterface(unittest.TestCase):
             self.fail("foo")
         except self.failureException:
             self.result.addFailure(self.test, sys.exc_info())
-        self.assertEquals(False, self.result.wasSuccessful())
+        self.assertEqual(False, self.result.wasSuccessful())
 
 
 
@@ -1150,7 +1175,7 @@ class TestReporter(TestReporterInterface):
 
 
 
-class TestSafeStream(unittest.TestCase):
+class TestSafeStream(unittest.SynchronousTestCase):
     def test_safe(self):
         """
         Test that L{reporter.SafeStream} successfully write to its original
@@ -1198,8 +1223,8 @@ class TestSubunitReporter(TestReporterInterface):
         subunitReturn = getattr(subunitClient, methodName)(*args, **kwargs)
         subunitOutput = stream.getvalue()
         reporterReturn = getattr(self.result, methodName)(*args, **kwargs)
-        self.assertEquals(subunitReturn, reporterReturn)
-        self.assertEquals(subunitOutput, self.stream.getvalue())
+        self.assertEqual(subunitReturn, reporterReturn)
+        self.assertEqual(subunitOutput, self.stream.getvalue())
 
 
     def removeMethod(self, klass, methodName):
@@ -1242,7 +1267,7 @@ class TestSubunitReporter(TestReporterInterface):
         self.stream.truncate(0)
         self.result.addSuccess(self.test)
         successOutput = self.stream.getvalue()
-        self.assertEquals(successOutput, expectedFailureOutput)
+        self.assertEqual(successOutput, expectedFailureOutput)
 
 
     def test_subunitWithoutAddSkipInstalled(self):
@@ -1256,7 +1281,7 @@ class TestSubunitReporter(TestReporterInterface):
         self.stream.truncate(0)
         self.result.addSuccess(self.test)
         successOutput = self.stream.getvalue()
-        self.assertEquals(successOutput, skipOutput)
+        self.assertEqual(successOutput, skipOutput)
 
 
     def test_addExpectedFailurePassedThrough(self):
@@ -1277,7 +1302,7 @@ class TestSubunitReporter(TestReporterInterface):
         except ZeroDivisionError:
             exc_info = sys.exc_info()
             self.result.addExpectedFailure(self.test, exc_info, 'todo')
-        self.assertEquals(addExpectedFailureCalls, [(self.test, exc_info)])
+        self.assertEqual(addExpectedFailureCalls, [(self.test, exc_info)])
 
 
     def test_addSkipSendsSubunitAddSkip(self):
@@ -1294,7 +1319,7 @@ class TestSubunitReporter(TestReporterInterface):
         # subunit has addSkip.
         self.result._subunit.addSkip = addSkip
         self.result.addSkip(self.test, 'reason')
-        self.assertEquals(addSkipCalls, [(self.test, 'reason')])
+        self.assertEqual(addSkipCalls, [(self.test, 'reason')])
 
 
     def test_doneDoesNothing(self):
@@ -1303,7 +1328,7 @@ class TestSubunitReporter(TestReporterInterface):
         of results is everything. Thus, done() does nothing.
         """
         self.result.done()
-        self.assertEquals('', self.stream.getvalue())
+        self.assertEqual('', self.stream.getvalue())
 
 
     def test_startTestSendsSubunitStartTest(self):
@@ -1359,7 +1384,7 @@ class TestSubunitReporter(TestReporterInterface):
         subunitClient.addSuccess(self.test)
         subunitOutput = stream.getvalue()
         self.result.addUnexpectedSuccess(self.test, 'todo')
-        self.assertEquals(subunitOutput, self.stream.getvalue())
+        self.assertEqual(subunitOutput, self.stream.getvalue())
 
 
     def test_loadTimeErrors(self):
@@ -1375,7 +1400,7 @@ class TestSubunitReporter(TestReporterInterface):
 
 
 
-class TestSubunitReporterNotInstalled(unittest.TestCase):
+class TestSubunitReporterNotInstalled(unittest.SynchronousTestCase):
     """
     Test behaviour when the subunit reporter is not installed.
     """
@@ -1388,7 +1413,7 @@ class TestSubunitReporterNotInstalled(unittest.TestCase):
         stream = StringIO.StringIO()
         self.patch(reporter, 'TestProtocolClient', None)
         e = self.assertRaises(Exception, reporter.SubunitReporter, stream)
-        self.assertEquals("Subunit not available", str(e))
+        self.assertEqual("Subunit not available", str(e))
 
 
 
@@ -1429,7 +1454,7 @@ class LoggingReporter(reporter.Reporter):
 
 
 
-class TestAdaptedReporter(unittest.TestCase):
+class TestAdaptedReporter(unittest.SynchronousTestCase):
     """
     L{reporter._AdaptedReporter} is a reporter wrapper that wraps all of the
     tests it receives before passing them on to the original reporter.
@@ -1444,7 +1469,8 @@ class TestAdaptedReporter(unittest.TestCase):
 
 
     def assertWrapped(self, wrappedResult, test):
-        self.assertEqual(wrappedResult._originalReporter.test, self._testAdapter(test))
+        self.assertEqual(wrappedResult._originalReporter.test,
+                         self._testAdapter(test))
 
 
     def getFailure(self, exceptionInstance):
@@ -1485,7 +1511,8 @@ class TestAdaptedReporter(unittest.TestCase):
         """
         C{addSkip} wraps its test with the provided adapter.
         """
-        self.wrappedResult.addSkip(self, self.getFailure(SkipTest('no reason')))
+        self.wrappedResult.addSkip(
+            self, self.getFailure(SkipTest('no reason')))
         self.assertWrapped(self.wrappedResult, self)
 
 
@@ -1540,7 +1567,7 @@ class FakeStream(object):
 
 
 
-class AnsiColorizerTests(unittest.TestCase):
+class AnsiColorizerTests(unittest.SynchronousTestCase):
     """
     Tests for L{reporter._AnsiColorizer}.
     """
@@ -1595,7 +1622,7 @@ class AnsiColorizerTests(unittest.TestCase):
         self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
         self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
 
-        self.assertEquals(sys.modules['curses'].setUp, 1)
+        self.assertEqual(sys.modules['curses'].setUp, 1)
 
 
     def test_supportedTigetNumWrongError(self):
@@ -1646,4 +1673,4 @@ class AnsiColorizerTests(unittest.TestCase):
 
         sys.modules['curses'] = fakecurses()
         self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
-        self.assertEquals(sys.modules['curses'].setUp, 1)
+        self.assertEqual(sys.modules['curses'].setUp, 1)
